@@ -1,64 +1,27 @@
 __doc__ = """Create block-structure class for collection of Cosserat rod systems."""
 import numpy as np
 from typing import Sequence
-
+from elastica.memory_block.memory_block_rod_base import (
+    MemoryBlockRodBase,
+    make_block_memory_metadata,
+    make_block_memory_periodic_boundary_metadata,
+)
 from elastica.rod.data_structures import _RodSymplecticStepperMixin
 from elastica.reset_functions_for_block_structure import _reset_scalar_ghost
-from elastica.rod.cosserat_rod import CosseratRod
+from elastica.rod.cosserat_rod import (
+    CosseratRod,
+    _compute_sigma_kappa_for_blockstructure,
+)
+from elastica._synchronize_periodic_boundary import (
+    _synchronize_periodic_boundary_of_vector_collection,
+    _synchronize_periodic_boundary_of_scalar_collection,
+    _synchronize_periodic_boundary_of_matrix_collection,
+)
 
 
-def make_block_memory_metadata(n_elems_in_rods):
-    """
-    This function, takes number of elements of each rod as an numpy array and computes,
-    ghost nodes, elements and voronoi element indexes and numbers and returns it.
-
-    Parameters
-    ----------
-    n_elems_in_rods
-
-    Returns
-    -------
-
-    """
-    n_nodes_in_rods = n_elems_in_rods + 1
-    n_voronois_in_rods = n_elems_in_rods - 1
-
-    n_rods = n_elems_in_rods.shape[0]
-
-    # Gap between two rods have one ghost node
-    # n_nodes_with_ghosts = np.sum(n_nodes_in_rods) + (n_rods - 1)
-    # Gap between two rods have two ghost elements : comes out to n_nodes_with_ghosts - 1
-    n_elems_with_ghosts = np.sum(n_elems_in_rods) + 2 * (n_rods - 1)
-    # Gap between two rods have three ghost voronois : comes out to n_nodes_with_ghosts - 2
-    # n_voronoi_with_ghosts = np.sum(n_voronois_in_rods) + 3 * (n_rods - 1)
-
-    # To be nulled
-    ghost_nodes_idx = np.zeros(((n_rods - 1),), dtype=np.int64)
-    ghost_nodes_idx[:] = n_nodes_in_rods[:-1]
-    ghost_nodes_idx = np.cumsum(ghost_nodes_idx)
-    # Add [0, 1, 2, ... n_rods-2] to the ghost_nodes idx to accommodate miscounting
-    ghost_nodes_idx += np.arange(0, n_rods - 1, dtype=np.int64)
-
-    ghost_elems_idx = np.zeros((2 * (n_rods - 1),), dtype=np.int64)
-    ghost_elems_idx[::2] = n_elems_in_rods[:-1]
-    ghost_elems_idx[1::2] = 1
-    ghost_elems_idx = np.cumsum(ghost_elems_idx)
-    # Add [0, 0, 1, 1, 2, 2, ... n_rods-2, n_rods-2] to the ghost_elems idx to accommodate miscounting
-    ghost_elems_idx += np.repeat(np.arange(0, n_rods - 1, dtype=np.int64), 2)
-
-    ghost_voronoi_idx = np.zeros((3 * (n_rods - 1),), dtype=np.int64)
-    ghost_voronoi_idx[::3] = n_voronois_in_rods[:-1]
-    ghost_voronoi_idx[1::3] = 1
-    ghost_voronoi_idx[2::3] = 1
-    ghost_voronoi_idx = np.cumsum(ghost_voronoi_idx)
-    # Add [0, 0, 0, 1, 1, 1, 2, 2, 2, ... n_rods-2, n_rods-2, n_rods-2] to the ghost_voronoi idx
-    # to accommodate miscounting
-    ghost_voronoi_idx += np.repeat(np.arange(0, n_rods - 1, dtype=np.int64), 3)
-
-    return n_elems_with_ghosts, ghost_nodes_idx, ghost_elems_idx, ghost_voronoi_idx
-
-
-class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
+class MemoryBlockCosseratRod(
+    MemoryBlockRodBase, CosseratRod, _RodSymplecticStepperMixin
+):
     """
     Memory block class for Cosserat rod equations. This class is derived from Cosserat Rod class in order to inherit
     the methods of Cosserat rod class. This class takes the cosserat rod object (systems) and creates big
@@ -68,9 +31,41 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
     TODO: need more documentation!
     """
 
-    def __init__(self, systems: Sequence):
+    def __init__(self, systems: Sequence, system_idx_list):
 
-        self.n_elems_in_rods = np.array([x.n_elems for x in systems], dtype=np.int64)
+        # separate straight and ring rods
+        system_straight_rod = []
+        system_ring_rod = []
+        system_idx_list_ring_rod = []
+        system_idx_list_straight_rod = []
+        for k, system_to_be_added in enumerate(systems):
+
+            if system_to_be_added.ring_rod_flag:
+                system_ring_rod.append(system_to_be_added)
+                system_idx_list_ring_rod.append(system_idx_list[k])
+                self.ring_rod_flag = True
+            else:
+                system_straight_rod.append(system_to_be_added)
+                system_idx_list_straight_rod.append(system_idx_list[k])
+
+        # Sorted systems
+        systems = system_straight_rod + system_ring_rod
+        self.system_idx_list = np.array(
+            system_idx_list_straight_rod + system_idx_list_ring_rod, dtype=np.int64
+        )
+
+        n_elems_straight_rods = np.array(
+            [x.n_elems for x in system_straight_rod], dtype=np.int64
+        )
+        n_elems_ring_rods = np.array(
+            [x.n_elems for x in system_ring_rod], dtype=np.int64
+        )
+
+        n_straight_rods = len(system_straight_rod)
+        n_ring_rods = len(system_ring_rod)
+
+        # self.n_elems_in_rods = np.array([x.n_elems for x in systems], dtype=np.int)
+        self.n_elems_in_rods = np.hstack((n_elems_straight_rods, n_elems_ring_rods + 2))
         self.n_rods = len(systems)
         (
             self.n_elems,
@@ -97,6 +92,90 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
             (self.ghost_voronoi_idx[::3], self.n_voronoi)
         )
 
+        # Periodic boundaries are only used if there is a ring rod present in the simulation, otherwise below
+        # arrays are empty.
+        (
+            _,
+            self.periodic_boundary_nodes_idx,
+            self.periodic_boundary_elems_idx,
+            self.periodic_boundary_voronoi_idx,
+        ) = make_block_memory_periodic_boundary_metadata(n_elems_ring_rods)
+
+        """
+        If there are ring rods appended to simulation, then start and end idx of rod nodes, elements and voronoi
+        have to be updated, to accommodate the periodic boundaries. Periodic boundaries can only accessed by
+        memory block  and rods does not have access to periodic boundaries.
+        """
+        if n_ring_rods != 0:
+            """
+            Number of nodes, elements and voronoi of the ring rod is same. When user wants to access to the rod, they
+            will also see that. However, in order to be compatible with block structure implementation, it has to be
+            n_nodes=n_elems+1 and n_voronoi=n_elems-1. Thus, we add these periodic nodes, element and voronoi at the
+            end of the rod. We need 3 periodic nodes,  2 periodic elements and 1 periodic voronoi.
+            Below you will see some magic numbers such as 1, 2, and 3. These are related with the number of periodic
+            nodes, element and voronoi.
+            For example if user sets 50 elements for one ring rod with 50 elements then we need two periodic element
+            at the start and end of the rod. So there are total of 52 elements on memory block including periodic ones.
+            In our numerical method n_nodes=n_elems+1 and n_voronoi=n_elems-1 so there are 53 nodes and 51 voronoi.
+            This requires 3 additional periodic nodes and 1 additional periodic voronoi. We add one of this periodic
+            node at the start of the rod and two at the end of the rod.
+
+            So below magic number 3 is coming from having extra 3 periodic nodes for each ring rod.
+                self.start_idx_in_rod_nodes[:] = (
+                    self.periodic_boundary_nodes_idx[0, 0::3] + 1
+                )
+
+            Same idea is used for elements and voronoi as well.
+            """
+            if n_straight_rods != 0:
+                # Here the idea of adding ghost nodes, elems and voronoi of straight rods is that, in memory block
+                # we place first straight rods, then ring rods.
+                # TODO: in future consider a better implementation for packing problem.
+                # +1 is because we want to start from next idx, where periodic boundary starts
+                self.periodic_boundary_nodes_idx += (
+                    self.ghost_nodes_idx[n_straight_rods - 1] + 1
+                )
+                self.periodic_boundary_elems_idx += (
+                    self.ghost_elems_idx[1::2][n_straight_rods - 1] + 1
+                )
+                self.periodic_boundary_voronoi_idx += (
+                    self.ghost_voronoi_idx[2::3][n_straight_rods - 1] + 1
+                )
+
+                # Compute the start and end of the rod nodes again. This time, boundary cells are added.
+                self.start_idx_in_rod_nodes[n_straight_rods:] = (
+                    self.periodic_boundary_nodes_idx[0, 0::3] + 1
+                )
+                self.end_idx_in_rod_nodes[
+                    n_straight_rods:
+                ] = self.periodic_boundary_nodes_idx[0, 1::3]
+
+                self.start_idx_in_rod_elems[n_straight_rods:] = (
+                    self.periodic_boundary_elems_idx[0, 0::2] + 1
+                )
+                self.end_idx_in_rod_elems[
+                    n_straight_rods:
+                ] = self.periodic_boundary_elems_idx[0, 1::2]
+
+                self.start_idx_in_rod_voronoi[n_straight_rods:] = (
+                    self.periodic_boundary_voronoi_idx[0, :] + 1
+                )
+            else:
+                # Compute the start and end of the rod nodes again. This time, boundary cells are added.
+                self.start_idx_in_rod_nodes[:] = (
+                    self.periodic_boundary_nodes_idx[0, 0::3] + 1
+                )
+                self.end_idx_in_rod_nodes[:] = self.periodic_boundary_nodes_idx[0, 1::3]
+
+                self.start_idx_in_rod_elems[:] = (
+                    self.periodic_boundary_elems_idx[0, 0::2] + 1
+                )
+                self.end_idx_in_rod_elems[:] = self.periodic_boundary_elems_idx[0, 1::2]
+
+                self.start_idx_in_rod_voronoi[:] = (
+                    self.periodic_boundary_voronoi_idx[0, :] + 1
+                )
+
         # Allocate block structure using system collection.
         self.allocate_block_variables_in_nodes(systems)
         self.allocate_block_variables_in_elements(systems)
@@ -109,6 +188,30 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
         _reset_scalar_ghost(self.rest_lengths, self.ghost_elems_idx, 1.0)
         _reset_scalar_ghost(self.rest_voronoi_lengths, self.ghost_voronoi_idx, 1.0)
 
+        # Compute strains for the block
+        _compute_sigma_kappa_for_blockstructure(self)
+
+        # If n_elems_with_boundary defined and passed with kwargs, then this rod is ring and we need to know
+        # how many boundary elements is rod containing.
+        if n_ring_rods != 0:
+            for sys_idx, system_to_be_added in enumerate(system_ring_rod):
+                if np.count_nonzero(system_to_be_added.rest_sigma) == 0:
+                    # Ring rod has to have non-zero rest sigma. If user did not set something, then Elastica will
+                    # calculate it.
+                    system_to_be_added.rest_sigma[:] = system_to_be_added.sigma[:]
+                if np.count_nonzero(system_to_be_added.rest_kappa) == 0:
+                    # Ring rod has to have non-zero rest kappa. If user did not set something, then Elastica will
+                    # calculate it.
+                    system_to_be_added.rest_kappa[:] = system_to_be_added.kappa[:]
+
+            # We update periodic elements and voronoi because they are used in difference and trapezoidal kernels.
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.rest_sigma, self.periodic_boundary_elems_idx
+            )
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.rest_kappa, self.periodic_boundary_voronoi_idx
+            )
+
         # Initialize the mixin class for symplectic time-stepper.
         _RodSymplecticStepperMixin.__init__(self)
 
@@ -129,10 +232,7 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
 
         # Things in nodes that are scalars
         #             0 ("mass", float64[:]),
-        map_scalar_dofs_in_rod_nodes = {
-            "mass": 0,
-            "dissipation_constant_for_forces": 1,
-        }
+        map_scalar_dofs_in_rod_nodes = {"mass": 0}
         self.scalar_dofs_in_rod_nodes = np.zeros(
             (len(map_scalar_dofs_in_rod_nodes), self.n_nodes)
         )
@@ -149,18 +249,20 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic node boundaries
+            _synchronize_periodic_boundary_of_scalar_collection(
+                self.__dict__[k], self.periodic_boundary_nodes_idx
+            )
 
         # Things in nodes that are vectors
         #             0 ("position_collection", float64[:, :]),
         #             1 ("internal_forces", float64[:, :]),
         #             2 ("external_forces", float64[:, :]),
-        #             3 ("damping_forces", float64[:, :]),
         # 6 in total
         map_vector_dofs_in_rod_nodes = {
             "position_collection": 0,
             "internal_forces": 1,
             "external_forces": 2,
-            "damping_forces": 3,
         }
         self.vector_dofs_in_rod_nodes = np.zeros(
             (len(map_vector_dofs_in_rod_nodes), 3 * self.n_nodes)
@@ -178,6 +280,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic node boundaries
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.__dict__[k], self.periodic_boundary_nodes_idx
+            )
 
         # Things in nodes that are matrices
         # Null set
@@ -205,8 +311,6 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
         #             4 ("rest_lengths", float64[:]),
         #             5 ("dilatation", float64[:]),
         #             6 ("dilatation_rate", float64[:]),
-        #             7 ("dissipation_constant_for_forces", float64[:]),
-        #             8 ("dissipation_constant_for_torques", float64[:])
         map_scalar_dofs_in_rod_elems = {
             "radius": 0,
             "volume": 1,
@@ -215,7 +319,6 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
             "rest_lengths": 4,
             "dilatation": 5,
             "dilatation_rate": 6,
-            "dissipation_constant_for_torques": 7,
         }
         self.scalar_dofs_in_rod_elems = np.zeros(
             (len(map_scalar_dofs_in_rod_elems), self.n_elems)
@@ -233,6 +336,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic element boundaries
+            _synchronize_periodic_boundary_of_scalar_collection(
+                self.__dict__[k], self.periodic_boundary_elems_idx
+            )
 
         # Things in elements that are vectors
         #             0 ("tangents", float64[:, :]),
@@ -240,7 +347,6 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
         #             2 ("rest_sigma", float64[:, :]),
         #             3 ("internal_torques", float64[:, :]),
         #             4 ("external_torques", float64[:, :]),
-        #             5 ("damping_torques", float64[:, :]),
         #             6 ("internal_stress", float64[:, :]),
         map_vector_dofs_in_rod_elems = {
             "tangents": 0,
@@ -248,8 +354,7 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
             "rest_sigma": 2,
             "internal_torques": 3,
             "external_torques": 4,
-            "damping_torques": 5,
-            "internal_stress": 6,
+            "internal_stress": 5,
         }
         self.vector_dofs_in_rod_elems = np.zeros(
             (len(map_vector_dofs_in_rod_elems), 3 * self.n_elems)
@@ -267,6 +372,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic element boundaries
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.__dict__[k], self.periodic_boundary_elems_idx
+            )
 
         # Things in elements that are matrices
         #             0 ("director_collection", float64[:, :, :]),
@@ -295,6 +404,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic element boundaries
+            _synchronize_periodic_boundary_of_matrix_collection(
+                self.__dict__[k], self.periodic_boundary_elems_idx
+            )
 
     def allocate_blocks_variables_in_voronoi(self, systems: Sequence):
         """
@@ -334,6 +447,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic voronoi boundaries
+            _synchronize_periodic_boundary_of_scalar_collection(
+                self.__dict__[k], self.periodic_boundary_voronoi_idx
+            )
 
         # Things in voronoi that are vectors
         #             0 ("kappa", float64[:, :]),
@@ -360,6 +477,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic voronoi boundaries
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.__dict__[k], self.periodic_boundary_voronoi_idx
+            )
 
         # Things in voronoi that are matrices
         #             0 ("bend_matrix", float64[:, :, :]),
@@ -381,6 +502,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic voronoi boundaries
+            _synchronize_periodic_boundary_of_matrix_collection(
+                self.__dict__[k], self.periodic_boundary_voronoi_idx
+            )
 
     def allocate_blocks_variables_for_symplectic_stepper(self, systems: Sequence):
         """
@@ -456,6 +581,10 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic node boundaries
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.__dict__[k], self.periodic_boundary_nodes_idx
+            )
 
         # Copy systems variables on nodes to block structure
         map_rate_collection_dofs_in_rod_elems = {
@@ -470,3 +599,7 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
                 system.__dict__[k] = np.ndarray.view(
                     self.__dict__[k][..., start_idx:end_idx]
                 )
+            # synchronize the periodic node boundaries
+            _synchronize_periodic_boundary_of_vector_collection(
+                self.__dict__[k], self.periodic_boundary_elems_idx
+            )
