@@ -1,8 +1,3 @@
-__all__ = [
-    "_reset_scalar_ghost",
-    "_reset_vector_ghost",
-]
-
 from elastica.reset_functions_for_block_structure._reset_ghost_vector_or_scalar import (
     _reset_scalar_ghost,
     _reset_vector_ghost,
