@@ -1,7 +1,5 @@
 __doc__ = """ Rod classes and implementation details """
-__all__ = ["CosseratRod"]
 
-import typing
 
 import numpy as np
 import functools
@@ -22,6 +20,7 @@ from elastica._calculus import (
     _difference,
     _average,
 )
+from typing import Optional
 
 position_difference_kernel = _difference
 position_average = _average
@@ -32,6 +31,40 @@ def _get_z_vector():
     return np.array([0.0, 0.0, 1.0]).reshape(3, -1)
 
 
+def _compute_sigma_kappa_for_blockstructure(memory_block):
+    """
+    This function is a wrapper to call functions which computes shear stretch, strain and bending twist and strain.
+
+    Parameters
+    ----------
+    memory_block : object
+
+    Returns
+    -------
+
+    """
+    _compute_shear_stretch_strains(
+        memory_block.position_collection,
+        memory_block.volume,
+        memory_block.lengths,
+        memory_block.tangents,
+        memory_block.radius,
+        memory_block.rest_lengths,
+        memory_block.rest_voronoi_lengths,
+        memory_block.dilatation,
+        memory_block.voronoi_dilatation,
+        memory_block.director_collection,
+        memory_block.sigma,
+    )
+
+    # Compute bending twist strains for the block
+    _compute_bending_twist_strains(
+        memory_block.director_collection,
+        memory_block.rest_voronoi_lengths,
+        memory_block.kappa,
+    )
+
+
 class CosseratRod(RodBase, KnotTheory):
     """
     Cosserat Rod class. This is the preferred class for rods because it is derived from some
@@ -77,13 +110,6 @@ class CosseratRod(RodBase, KnotTheory):
         inv_mass_second_moment_of_inertia: numpy.ndarray
             3D (dim, dim, n_elems) array containing data with 'float' type.
             Rod element inverse mass moment of inertia.
-        dissipation_constant_for_forces: numpy.ndarray
-            1D (n_elems) array containing data with 'float' type.
-            Rod element dissipation coefficient (nu).
-        dissipation_constant_for_torques: numpy.ndarray
-            1D (n_elems) array containing data with 'float' type.
-            Rod element dissipation (nu).
-            Can be customized by passing 'nu_for_torques'.
         rest_voronoi_lengths: numpy.ndarray
             1D (n_voronoi) array containing data with 'float' type.
             Rod lengths on the voronoi domain at the rest configuration.
@@ -136,8 +162,6 @@ class CosseratRod(RodBase, KnotTheory):
         density,
         volume,
         mass,
-        dissipation_constant_for_forces,
-        dissipation_constant_for_torques,
         internal_forces,
         internal_torques,
         external_forces,
@@ -155,8 +179,7 @@ class CosseratRod(RodBase, KnotTheory):
         rest_kappa,
         internal_stress,
         internal_couple,
-        damping_forces,
-        damping_torques,
+        ring_rod_flag,
     ):
         self.n_elems = n_elements
         self.position_collection = position
@@ -173,8 +196,6 @@ class CosseratRod(RodBase, KnotTheory):
         self.density = density
         self.volume = volume
         self.mass = mass
-        self.dissipation_constant_for_forces = dissipation_constant_for_forces
-        self.dissipation_constant_for_torques = dissipation_constant_for_torques
         self.internal_forces = internal_forces
         self.internal_torques = internal_torques
         self.external_forces = external_forces
@@ -192,28 +213,30 @@ class CosseratRod(RodBase, KnotTheory):
         self.rest_kappa = rest_kappa
         self.internal_stress = internal_stress
         self.internal_couple = internal_couple
-        self.damping_forces = damping_forces
-        self.damping_torques = damping_torques
+        self.ring_rod_flag = ring_rod_flag
 
-        # Compute shear stretch and strains.
-        _compute_shear_stretch_strains(
-            self.position_collection,
-            self.volume,
-            self.lengths,
-            self.tangents,
-            self.radius,
-            self.rest_lengths,
-            self.rest_voronoi_lengths,
-            self.dilatation,
-            self.voronoi_dilatation,
-            self.director_collection,
-            self.sigma,
-        )
+        if not self.ring_rod_flag:
+            # For ring rod there are no periodic elements so below code won't run.
+            # We add periodic elements at the memory block construction.
+            # Compute shear stretch and strains.
+            _compute_shear_stretch_strains(
+                self.position_collection,
+                self.volume,
+                self.lengths,
+                self.tangents,
+                self.radius,
+                self.rest_lengths,
+                self.rest_voronoi_lengths,
+                self.dilatation,
+                self.voronoi_dilatation,
+                self.director_collection,
+                self.sigma,
+            )
 
-        # Compute bending twist strains
-        _compute_bending_twist_strains(
-            self.director_collection, self.rest_voronoi_lengths, self.kappa
-        )
+            # Compute bending twist strains
+            _compute_bending_twist_strains(
+                self.director_collection, self.rest_voronoi_lengths, self.kappa
+            )
 
     @classmethod
     def straight_rod(
@@ -225,9 +248,9 @@ class CosseratRod(RodBase, KnotTheory):
         base_length: float,
         base_radius: float,
         density: float,
-        nu: float,
+        *,
+        nu: Optional[float] = None,
         youngs_modulus: float,
-        *args,
         **kwargs,
     ):
         """
@@ -243,7 +266,8 @@ class CosseratRod(RodBase, KnotTheory):
         Parameters
         ----------
         n_elements : int
-            Number of element. Must be greater than 3. Generarally recommended to start with 40-50, and adjust the resolution.
+            Number of element. Must be greater than 3.
+            Generally recommended to start with 40-50, and adjust the resolution.
         start : NDArray[3, float]
             Starting coordinate in 3D
         direction : NDArray[3, float]
@@ -260,9 +284,6 @@ class CosseratRod(RodBase, KnotTheory):
             Damping coefficient for Rayleigh damping
         youngs_modulus : float
             Young's modulus
-        *args : tuple
-            Additional arguments should be passed as keyward arguments.
-            (e.g. shear_modulus, poisson_ratio)
         **kwargs : dict, optional
             The "position" and/or "directors" can be overrided by passing "position" and "directors" argument. Remember, the shape of the "position" is (3,n_elements+1) and the shape of the "directors" is (3,3,n_elements).
 
@@ -272,6 +293,17 @@ class CosseratRod(RodBase, KnotTheory):
 
         """
 
+        if nu is not None:
+            raise ValueError(
+                # Remove the option to set internal nu inside, beyond v0.4.0
+                "The option to set damping coefficient (nu) for the rod during rod\n"
+                "initialisation is now deprecated. Instead, for adding damping to rods,\n"
+                "please derive your simulation class from the add-on Damping mixin class.\n"
+                "For reference see the class elastica.dissipation.AnalyticalLinearDamper(),\n"
+                "and for usage check examples/axial_stretching.py"
+            )
+        # Straight rod is not ring rod set flag to false
+        ring_rod_flag = False
         (
             n_elements,
             position,
@@ -288,8 +320,6 @@ class CosseratRod(RodBase, KnotTheory):
             density,
             volume,
             mass,
-            dissipation_constant_for_forces,
-            dissipation_constant_for_torques,
             internal_forces,
             internal_torques,
             external_forces,
@@ -307,19 +337,16 @@ class CosseratRod(RodBase, KnotTheory):
             rest_kappa,
             internal_stress,
             internal_couple,
-            damping_forces,
-            damping_torques,
         ) = allocate(
             n_elements,
-            start,
             direction,
             normal,
             base_length,
             base_radius,
             density,
-            nu,
             youngs_modulus,
-            *args,
+            rod_origin_position=start,
+            ring_rod_flag=ring_rod_flag,
             **kwargs,
         )
 
@@ -339,8 +366,6 @@ class CosseratRod(RodBase, KnotTheory):
             density,
             volume,
             mass,
-            dissipation_constant_for_forces,
-            dissipation_constant_for_torques,
             internal_forces,
             internal_torques,
             external_forces,
@@ -358,8 +383,154 @@ class CosseratRod(RodBase, KnotTheory):
             rest_kappa,
             internal_stress,
             internal_couple,
-            damping_forces,
-            damping_torques,
+            ring_rod_flag,
+        )
+
+    @classmethod
+    def ring_rod(
+        cls,
+        n_elements: int,
+        ring_center_position: np.ndarray,
+        direction: np.ndarray,
+        normal: np.ndarray,
+        base_length: float,
+        base_radius: float,
+        density: float,
+        *,
+        nu: Optional[float] = None,
+        youngs_modulus: float,
+        **kwargs,
+    ):
+        """
+        Cosserat rod constructor for straight-rod geometry.
+
+
+        Notes
+        -----
+        Since we expect the Cosserat Rod to simulate soft rod, Poisson's ratio is set to 0.5 by default.
+        It is possible to give additional argument "shear_modulus" or "poisson_ratio" to specify extra modulus.
+
+
+        Parameters
+        ----------
+        n_elements : int
+            Number of element. Must be greater than 3. Generarally recommended to start with 40-50, and adjust the resolution.
+        ring_center_position : NDArray[3, float]
+            Center coordinate for ring rod in 3D
+        direction : NDArray[3, float]
+            Direction of the rod in 3D
+        normal : NDArray[3, float]
+            Normal vector of the rod in 3D
+        base_length : float
+            Total length of the rod
+        base_radius : float
+            Uniform radius of the rod
+        density : float
+            Density of the rod
+        nu : float
+            Damping coefficient for Rayleigh damping
+        youngs_modulus : float
+            Young's modulus
+        **kwargs : dict, optional
+            The "position" and/or "directors" can be overrided by passing "position" and "directors" argument. Remember, the shape of the "position" is (3,n_elements+1) and the shape of the "directors" is (3,3,n_elements).
+
+        Returns
+        -------
+        CosseratRod
+
+        """
+
+        if nu is not None:
+            raise ValueError(
+                # Remove the option to set internal nu inside, beyond v0.4.0
+                "The option to set damping coefficient (nu) for the rod during rod\n"
+                "initialisation is now deprecated. Instead, for adding damping to rods,\n"
+                "please derive your simulation class from the add-on Damping mixin class.\n"
+                "For reference see the class elastica.dissipation.AnalyticalLinearDamper(),\n"
+                "and for usage check examples/axial_stretching.py"
+            )
+        # Straight rod is not ring rod set flag to false
+        ring_rod_flag = True
+        (
+            n_elements,
+            position,
+            velocities,
+            omegas,
+            accelerations,
+            angular_accelerations,
+            directors,
+            radius,
+            mass_second_moment_of_inertia,
+            inv_mass_second_moment_of_inertia,
+            shear_matrix,
+            bend_matrix,
+            density_array,
+            volume,
+            mass,
+            internal_forces,
+            internal_torques,
+            external_forces,
+            external_torques,
+            lengths,
+            rest_lengths,
+            tangents,
+            dilatation,
+            dilatation_rate,
+            voronoi_dilatation,
+            rest_voronoi_lengths,
+            sigma,
+            kappa,
+            rest_sigma,
+            rest_kappa,
+            internal_stress,
+            internal_couple,
+        ) = allocate(
+            n_elements,
+            direction,
+            normal,
+            base_length,
+            base_radius,
+            density,
+            youngs_modulus,
+            rod_origin_position=ring_center_position,
+            ring_rod_flag=ring_rod_flag,
+            **kwargs,
+        )
+
+        return cls(
+            n_elements,
+            position,
+            velocities,
+            omegas,
+            accelerations,
+            angular_accelerations,
+            directors,
+            radius,
+            mass_second_moment_of_inertia,
+            inv_mass_second_moment_of_inertia,
+            shear_matrix,
+            bend_matrix,
+            density_array,
+            volume,
+            mass,
+            internal_forces,
+            internal_torques,
+            external_forces,
+            external_torques,
+            lengths,
+            rest_lengths,
+            tangents,
+            dilatation,
+            dilatation_rate,
+            voronoi_dilatation,
+            rest_voronoi_lengths,
+            sigma,
+            kappa,
+            rest_sigma,
+            rest_kappa,
+            internal_stress,
+            internal_couple,
+            ring_rod_flag,
         )
 
     def compute_internal_forces_and_torques(self, time):
@@ -390,9 +561,6 @@ class CosseratRod(RodBase, KnotTheory):
             self.rest_sigma,
             self.shear_matrix,
             self.internal_stress,
-            self.velocity_collection,
-            self.dissipation_constant_for_forces,
-            self.damping_forces,
             self.internal_forces,
             self.ghost_elems_idx,
         )
@@ -415,8 +583,6 @@ class CosseratRod(RodBase, KnotTheory):
             self.internal_couple,
             self.dilatation,
             self.dilatation_rate,
-            self.dissipation_constant_for_torques,
-            self.damping_torques,
             self.internal_torques,
             self.ghost_voronoi_idx,
         )
@@ -722,26 +888,6 @@ def _compute_internal_bending_twist_stresses_from_model(
     internal_couple[:] = _batch_matvec(bend_matrix, temp)
 
 
-@numba.njit(cache=True)
-def _compute_damping_forces(
-    damping_forces,
-    velocity_collection,
-    dissipation_constant_for_forces,
-):
-    """
-    Update <damping force> given <velocity>
-    """
-
-    # Internal damping forces.
-    blocksize = velocity_collection.shape[1]
-
-    for i in range(3):
-        for k in range(blocksize):
-            damping_forces[i, k] = (
-                dissipation_constant_for_forces[k] * velocity_collection[i, k]
-            )
-
-
 @numba.njit(cache=True)
 def _compute_internal_forces(
     position_collection,
@@ -758,14 +904,11 @@ def _compute_internal_forces(
     rest_sigma,
     shear_matrix,
     internal_stress,
-    velocity_collection,
-    dissipation_constant_for_forces,
-    damping_forces,
     internal_forces,
     ghost_elems_idx,
 ):
     """
-    Update <internal force> given <director, internal_stress, velocity and damping force>.
+    Update <internal force> given <director, internal_stress and velocity>.
     """
 
     # Compute n_l and cache it using internal_stress
@@ -801,33 +944,10 @@ def _compute_internal_forces(
                 )
 
     cosserat_internal_stress /= dilatation
-
-    _compute_damping_forces(
-        damping_forces,
-        velocity_collection,
-        dissipation_constant_for_forces,
+    internal_forces[:] = difference_kernel_for_block_structure(
+        cosserat_internal_stress, ghost_elems_idx
     )
 
-    internal_forces[:] = (
-        difference_kernel_for_block_structure(cosserat_internal_stress, ghost_elems_idx)
-        - damping_forces
-    )
-
-
-@numba.njit(cache=True)
-def _compute_damping_torques(
-    damping_torques, omega_collection, dissipation_constant_for_torques
-):
-    """
-    Update <damping torque> given <angular velocity>.
-    """
-    blocksize = omega_collection.shape[1]
-    for i in range(3):
-        for k in range(blocksize):
-            damping_torques[i, k] = (
-                dissipation_constant_for_torques[k] * omega_collection[i, k]
-            )
-
 
 @numba.njit(cache=True)
 def _compute_internal_torques(
@@ -848,8 +968,6 @@ def _compute_internal_torques(
     internal_couple,
     dilatation,
     dilatation_rate,
-    dissipation_constant_for_torques,
-    damping_torques,
     internal_torques,
     ghost_voronoi_idx,
 ):
@@ -908,10 +1026,6 @@ def _compute_internal_torques(
     # (J \omega_L / e^2) . (de/dt)
     unsteady_dilatation = J_omega_upon_e * dilatation_rate / dilatation
 
-    _compute_damping_torques(
-        damping_torques, omega_collection, dissipation_constant_for_torques
-    )
-
     blocksize = internal_torques.shape[1]
     for i in range(3):
         for k in range(blocksize):
@@ -921,7 +1035,6 @@ def _compute_internal_torques(
                 + shear_stretch_couple[i, k]
                 + lagrangian_transport[i, k]
                 + unsteady_dilatation[i, k]
-                - damping_torques[i, k]
             )
 
 
